// Copyright (c) FIRST and other WPILib contributors.
// Open Source Software; you can modify and/or share it under the terms of
// the WPILib BSD license file in the root directory of this project.

package frc.robot.commands.Limelight;
import static edu.wpi.first.units.Units.*;

import java.util.function.DoubleSupplier;

import com.ctre.phoenix6.swerve.SwerveModule.DriveRequestType;
import com.ctre.phoenix6.swerve.SwerveRequest;

import edu.wpi.first.wpilibj2.command.Command;
import frc.robot.TunerConstants.TunerConstants;
import frc.robot.subsystems.CommandSwerveDrivetrain;
import frc.robot.subsystems.Limelight3;

/* You should consider using the more terse Command factories API instead https://docs.wpilib.org/en/stable/docs/software/commandbased/organizing-command-based.html#defining-commands */
public class Forme3 extends Command {
  private Limelight3 limelight3;
  private double MaxSpeed = TunerConstants.kSpeedAt12Volts.in(MetersPerSecond);
  private double MaxAngularRate = RotationsPerSecond.of(0.75).in(RadiansPerSecond); // 3/4 of a rotation per second max angular velocity
  private CommandSwerveDrivetrain drivetrain = TunerConstants.createDrivetrain();
  private DoubleSupplier x;
  private DoubleSupplier y;
    /* Setting up bindings for necessary control of the swerve drive platform */
    private final SwerveRequest.FieldCentric drive = new SwerveRequest.FieldCentric()
            .withDeadband(MaxSpeed * 0.1).withRotationalDeadband(MaxAngularRate * 0.1) // Add a 10% deadband
            .withDriveRequestType(DriveRequestType.OpenLoopVoltage); // Use open-loop control for drive motors
  /** Creates a new Forme3. */
  public Forme3(Limelight3 limelight3,CommandSwerveDrivetrain drivetrain, DoubleSupplier x, DoubleSupplier y) {
    this.limelight3 = limelight3;
    this.drivetrain = drivetrain;
    this.x = x;
    this.y = y;
    addRequirements(limelight3,drivetrain);
    // Use addRequirements() here to declare subsystem dependencies.
  }

  // Called when the command is initially scheduled.
  @Override
  public void initialize() {
    limelight3.Forme();
  }

  // Called every time the scheduler runs while the command is scheduled.
  @Override
  public void execute() {
    double a = limelight3.getX();
    if(limelight3.getV() == true){
      drivetrain.setControl(drive.
      withRotationalRate(a/10).
      withVelocityX(x.getAsDouble()).
      withVelocityY(y.getAsDouble()));  
       System.out.println(a/10);
    }
    else{
      drivetrain.setControl(drive.
      withRotationalRate(0).
      withVelocityX(0).
      withVelocityY(0));
    }
  }

  // Called once the command ends or is interrupted.
  @Override
  public void end(boolean interrupted) {
    drivetrain.setControl(drive.
      withRotationalRate(0).
      withVelocityX(0).
      withVelocityY(0));
  }

  // Returns true when the command should end.
  @Override
  public boolean isFinished() {
    return false;
  }
}
